This workflow is intended for the analysis of paired end Illumina amplicon sequences generated on a MiSeq in a 2x300bp run. It was developed and tested using 16S V3-V4 amplicon data generated at [LGC](https://www.biosearchtech.com/services/sequencing/dna-sequencing/microbrial-research). The parameter settings in the [config file](https://git.io-warnemuende.de/bio_inf/workflow_templates/src/branch/master/Amplicon_dada2_MiSeq/config/config.yaml) are based on such input data. Further details about which aspects to consider when running dada2 are included in the [amplicon sequence analysis tutorial](https://git.io-warnemuende.de/bio_inf/tutorials_collection/wiki/Amplicon-sequence-analysis-using-dada2).


### Overview of workflow

<img src="https://git.io-warnemuende.de/bio_inf/workflow_templates/raw/branch/master/Amplicon_dada2_MiSeq/images/amplicon_dada2.png" width="60%" height="60%">


### Workflow configuration

Change the following parameters in the [config file](https://git.io-warnemuende.de/bio_inf/workflow_templates/src/branch/master/Amplicon_dada2_MiSeq/config/config.yaml) to adapt the workflow to your project.

General parameters:

* **project**: The project name will be used in the output files generated during the workflow. Ideally it should contain the ProjectID and optionally some other short meaningful descriptor. White spaces and special characters are not allowed.
* **wdir**: Location of the scripts for the analysis workflow (i.e. working directory). This is the place where the Snakefiles are located and the where you will start the workflow from.
* **sample_list**: File path to the table with sample names and input sequence file locations. This should be a tab-separated text file without a header, with sample names in the first column, the absolute path to the R1 and R2 read files in the second and third columns, respectively. Sequence files should be demultiplexed but not yet primer clipped or reoriented. They can be zipped (using any of the compression format supported by [cutadapt](https://cutadapt.readthedocs.io/en/stable/guide.html#compressed-files)).
* **adir**: Analysis data directory for the project. This is the top level directory, in which the sub-directories for ```Validated_data``` and ```Intermediate_results``` will be generated.
* **analysis**: Name for the version of the analysis run. This will be used to structure lower level directories in ```adir``` to make sure that output files are not mixed between re-runs of the workflow or between amplicon and shotgun analyses.
* **threads**, **sample_threads**, and **parallel_threads**: Number of threads to use by default, for processes that are running per sample (should correspond to the number of samples), and for other parallelized processes (e.g. denoising, chimera detection, taxonomic classification) which are rather independent of the number of samples.

Primer clipping parameters:

* **fwd**: Forward primer sequence. IUPAC degenerate code accepted. If your primer is anchored at the 5' end of the read, prepend the primer sequence with ```^```.
* **rev**: Reverse primer sequence. IUPAC degenerate code accepted. If your primer is anchored at the 5' end of the read, prepend the primer sequence with ```^```.
* **ofwd**: Required overlap to detect the forward primer. Recommendation: primer length - 1.
* **orev**: Required overlap to detect the reverse primer. Recommendation: primer length - 1.
* **error**: Percentage allowed mismatches to primer sequence.
error: 0.16

Ranges for screening filtering and trimming parameters:

* **truncLen_R1_min** and **truncLen_R1_max**: Minimum and maximum truncation length to be applied to the R1 read. The truncation length for the R2 read will be calculated based on ```fragment_max - truncLen_R1 + 30```.
* **fragment_max**: Maximum expected fragment size. For the 16S V3-V4 region this is typically about 430bp.
* **maxEE_min** and **maxEE_max**: Range of values for the maximum expected errors (maxEE). maxEE will be incrementally increased with the R2 read leading, e.g. ```c(2, 2)```, ```c(2, 3)```, ```c(3, 3)```.

Final filtering and trimming parameters (maybe to be adjusted after parameter screening):

* **truncLen_R1**: Truncation length for R1 read.
* **truncLen_R2**: Truncation length for R2 read.
* **maxEE_R1**: Maximum expected errors for R1 read.
* **maxEE_R2**: Maximum expected errors for R2 read.
* **min_len** and **max_len**: Range of expected fragment lengths, i.a. amplicon without primer sequences. All sequences outside this range will be discarded after merging and chimera detection.

Parameters for the taxonomic classification:

* **silva_dada2**: Location of silva training data set for dada2-based taxonomic classification.
* **min_boot**: Bootstrap cut-off for dada2-based taxonomic classification.
* **silva_blast**: Location of silva blast DB.
* **min_blast**: Cut-off for filtering blast results (applied to the average of pident and qcovs).
* **silva_acc2tax**: Location of silva accesion-to-taxonomy map.
* **silva_taxrank**: Location of silva taxon-to-rank map.
* **lineage**: Taxonomic domains to keep in the output. If more than one, use comma separated list, e.g.```"Bacteria,Archaea"```


### Running the workflow

The workflow consists of 3 different elements, after each of which a quick manual check of the output data may be required. The 3 elements can be run (in that order) as follows:

```
# Prepare environment
cd /bio/Common_repositories/workflow_templates/Amplicon_dada2_MiSeq/
conda activate snakemake

# 1: Primer clipping and parameter screening for filtering settings
snakemake -s Snakefile_trimming --use-conda --cores N

# 2: Denoising
snakemake -s Snakefile_denoising --use-conda --cores N

# 3: Taxonomic classification
snakemake -s Snakefile_classification --use-conda --cores N
```

Snakemake uses conda environments to run the required software. To avoid the creation of too many additional conda environment, please **always start this snakemake workflow in the Amplicon_dada2_MiSeq directory**. Snakefiles and config files can be copied and moved to project-specific directories to be adapted as required. Make sure to use the correct path to the config file in the Snakefile. The placeholder ```N``` in the above commands should be replaced by the maximum number of CPUs to use for parallelized tasks.

:warning: If you encounter any problems, please don't hesitate to open an issue here.


### Description of output files

The workflow will generate the following files and directories:

```
├── Intermediate_results
│   ├── asvtab_Test_dada2.txt                      # ASV table
│   ├── blastout_filt_Test_dada2.txt               # Filtered blast output
│   ├── blastout_Test_dada2.txt                    # Raw blast output
│   ├── ErrorProfiles_Test_dada2.pdf               # Convergence plot and error profiles
│   ├── Filtered/                                  # Directory with filtered and trimmed read files
│   │   
│   ├── Length_distribution_Test_dada2.txt         # Number of sequences for each ASV length
│   ├── Logfiles/                                  # Directory to collect the logfiles for each analysis step in the workflow
│   │   
│   ├── nseqs_all_Test_dada2.txt                   # Number of sequences at each analysis step
│   ├── nseqs_Test_dada2.txt                       # Number of sequences for demultiplexed and primer-clipped reads
│   ├── Parameter_screening_Test_dada2.pdf         # Visualization of results of parameter screening for filtering and trimming settings
│   ├── Parameter_screening_Test_dada2.txt         # Percentage retained sequences for different filtering and trimming settings
│   ├── QualityProfileFiltered_Test_dada2_R1.pdf   # Quality profiles of filtered and trimmed R1 reads
│   ├── QualityProfileFiltered_Test_dada2_R2.pdf   # Quality profiles of filtered and trimmed R2 reads
│   ├── QualityProfile_Test_dada2_R1.pdf           # Quality profiles of primer clipped R1 reads
│   ├── QualityProfile_Test_dada2_R2.pdf           # Quality profiles of primer clipped R2 reads
│   ├── seqs_Test_dada2.fasta                      # Fasta file of ASVs
│   ├── tax_blast_Test_dada2.txt                   # Taxonomic classification using blast (incl. qlen, pident, qcovs from blast output)
│   ├── tax_dada2_Test_dada2.txt                   # Taxonomic classification using dada2 assignTaxonomy()
│   ├── workspace_classification_Test_dada2.Rdata  # R workspace for taxonomic classification
│   └── workspace_denoising_Test_dada2.Rdata       # R workspace for denoising
└── Validated_data/                                # Primer clipped read files suitable for data archiving
```

The main output files are (in this example **Test_dada2** was set as project name in the config file): 

* **asvtab_Test_dada2.txt**: Sample-by-ASV table, in which samples are columns and ASVs are rows for better human readability. Sequences are used as rownames.
* **seqs_Test_dada2.txt**: ASV fasta file. Here, the sequences are named by dada2 as sq1...n (which appear as qseqid in the blast output).
* **tax_dada2_Test_dada2.txt**: Domain to genus classification. Sequences are used as rownames. Be aware that the dimensions of the taxonomy file may not match the ASV table since it is likely that not all ASV were classified. It may be necessary to subset the ASV table to only the classified sequences.

The other files are mainly diagnostic. I recommend to have a look at the ```QualityProfile``` and ```Parameter_screening``` files to confirm the filtering and trimming setting before starting the second snakemake command (and to check the ```QualityProfileFiltered``` files afterwards). Also confirm the number of sequences retained at each analysis step by consulting ```nseqs_all_Test_dada2.txt```. If anything does not seem right, there is further information provided in the various logfiles. If you want to repeat steps outside of the snakemake workflow, the R workspaces (including all objects created in R during the workflow) are saved and can be further used for customized analyses.

Examples of output files are provided in [example_output](https://git.io-warnemuende.de/bio_inf/workflow_templates/src/branch/master/Amplicon_dada2_MiSeq/example_output).


### Hints to prepare the ```sample_list``` file

As it can be quite tedious to manually enter the file path for a large number of samples, I strongly recommend using the ```ls -1d $PWD/*R1.fastq.gz > file_list.txt``` to first save a list of file paths that can then be pasted into the sample_list table. For data provided by LGC this is a bit more complicated, since there each sample has its own sub-directory. Here something like ```ls -1d $PWD/*/*R1_clipped.fastq.bz2``` (run in the AdapterClipped directory) may be appropriate. Alternatively, you can extract the sample list information directly from the sample_metadata.tsv under ```/bio/Raw_data/yourProjectID```. This can be done with the following command: ```cut -f35,36,37 sample_metadata.tsv | sed '1d'```. Make sure that there is no empty line at the end of the file.


### Expected fragment size ranges for common primer sets

The following are estimates of the length variation of popular amplicons that can be used to constrain the dada2 output. They may be used (with caution) as basis for selecting trimming parameters and cleaning the final sequence table.

* 16S V3-V4 (FWD:CCTAYGGGRBGCASCAG REV:GGACTACNNGGGTATCTAAT): 380 - 433bp (bacteria and archaea)
* 16S V3-V4 (FWD:CCTACGGGNGGCWGCAG REV:GACTACHVGGGTATCTAAKCC): 399 - 432bp (bacteria)
* 16S V4-V5 (FWD:GTGYCAGCMGCCGCGGTAA REV:CCGYCAATTYMTTTRAGTTT): 360 - 400bp (bacteria and archaea)
* 18S V4 (FWD:CCAGCASCYGCGGTAATTCC REV:ACTTTCGTTCTTGATYRATGA): 304 - 426bp


### Citation

When using this workflow, please cite:

* Callahan, B., McMurdie, P., Rosen, M. et al. DADA2: High-resolution sample inference from Illumina amplicon data. Nat Methods 13, 581–583 (2016). https://doi.org/10.1038/nmeth.3869
* Martin, M. Cutadapt removes adapter sequences from high-throughput sequencing reads. EMBnet.journal, [S.l.], v. 17, n. 1, p. pp. 10-12, may 2011. ISSN 2226-6089. Available at: http://journal.embnet.org/index.php/embnetjournal/article/view/200/479
* Camacho C., Coulouris G., Avagyan V., Ma N., Papadopoulos J., Bealer K., & Madden T.L. (2008) "BLAST+: architecture and applications." BMC Bioinformatics 10:421. doi: https://doi.org/10.1186/1471-2105-10-421
* R Core Team (2021). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.
* O. Tange (2011): GNU Parallel - The Command-Line Power Tool, ;login: The USENIX Magazine, February 2011:42-47.
* Mölder, F., Jablonski, K.P., Letcher, B., Hall, M.B., Tomkins-Tinch, C.H., Sochat, V., Forster, J., Lee, S., Twardziok, S.O., Kanitz, A., Wilm, A., Holtgrewe, M., Rahmann, S., Nahnsen, S., Köster, J., 2021. Sustainable data analysis with Snakemake. F1000Res 10, 33.
* Quast, C., Pruesse, E., Yilmaz, P., Gerken, J., Schweer, T., Yarza, P., … Glöckner, F. O. (2013). The SILVA ribosomal RNA gene database project: improved data processing and web-based tools. Nucleic Acids Research, 41(Database issue), D590-6. doi: https://doi.org/10.1093/nar/gks1219